@@ -44,6 +44,22 @@ print(AmericanOption("call", underlying=90, strike=100, div=0.1, riskFree=0.1,
                      maturity=0.1, vol=0.25), digits=5)
 
 
+# Discrete dividend
+## europeanoption.cpp:  call value ==  3.67
+## Reference pg. 253 - Hull 5th ed Exercise 12.8 - From QuantLib tests
+print(EuropeanOption("call", underlying=40, strike=40, div=0, riskFree=0.09,
+                     maturity=0.5, vol=0.3,
+                     discreteDividends = c(0.5, 0.5),
+                     discreteDividendsTimeUntil = c(2/12, 5/12)), digits=5)
+
+## americanoption.cpp:  call value == 3.72 (Hull) -- we show 3.75
+## Reference p. 256 - Hull 5th ed. Exercise 12.9 using (flawed) Roll, Geske, Whaley formula
+print(AmericanOption("call", underlying=40, strike=40, div=0, riskFree=0.09,
+                     maturity=0.5, vol=0.3,
+                     engine = "CrankNicolson",
+                     discreteDividends = c(0.5, 0.5),
+                     discreteDividendsTimeUntil = c(2/12, 5/12)), digits=5)
+
 ## barrier: down and out call == 9.0246
 print(BarrierOption("downout", barrier=95, rebate=3, type="call",
                     strike=90, underlying=100, div=0.04, riskF=0.08,
@@ -67,7 +83,7 @@ print( AsianOption("geometric", "put", underlying=80, strike=85, div=-0.03, risk
 bond <- list(faceAmount=100,issueDate=as.Date("2004-11-30"),
              maturityDate=as.Date("2008-11-30"), redemption=100 )
 
-dateparams <-list(settlementDays=1, calendar="us", businessDayConvention='Unadjusted')
+dateparams <-list(settlementDays=1, calendar="UnitedStates/GovernmentBond", businessDayConvention='Unadjusted')
 
 
 discountCurve.param <- list(tradeDate=as.Date('2002-2-15'),
@@ -94,39 +110,64 @@ FixedRateBondPriceByYield(,0.0307, 100000, as.Date("2004-11-30"), as.Date("2008-
 ## bond.cpp
 
 #Simple call with a flat curve
-bond <- list(faceAmount=100,
+bond <- list(settlementDays=1,
              issueDate=as.Date("2004-11-30"),
-             maturityDate=as.Date("2008-11-30"),
-             redemption=100, 
-             effectiveDate=as.Date("2004-11-30"))
-dateparams <- list(settlementDays=1,
-                   calendar="us", dayCounter = 'Thirty360', period=2, 
-                   businessDayConvention = 4, terminationDateConvention=4,
-                   dateGeneration=1, endOfMonth=1)
+             faceAmount=100,
+             accrualDayCounter='Thirty360',
+             paymentConvention='Unadjusted')
+schedule <- list(effectiveDate=as.Date("2004-11-30"),
+                 maturityDate=as.Date("2008-11-30"),
+                 period='Semiannual',
+                 calendar='UnitedStates/GovernmentBond',
+                 businessDayConvention='Unadjusted',
+                 terminationDateConvention='Unadjusted',
+                 dateGeneration='Forward',
+                 endOfMonth=1)
+calc=list(dayCounter='Actual360',
+          compounding='Compounded',
+          freq='Annual',
+          durationType='Modified')
 coupon.rate <- c(0.02875)
-                       
 params <- list(tradeDate=as.Date('2002-2-15'),
                settleDate=as.Date('2002-2-19'),
                dt=.25,
                interpWhat="discount",
                interpHow="loglinear")
-
 discountCurve.flat <- DiscountCurve(params, list(flat=0.05))
-FixedRateBond(bond, coupon.rate, discountCurve.flat, dateparams)
+FixedRateBond(bond,
+              coupon.rate,
+              schedule,
+              calc,
+              discountCurve=discountCurve.flat)
+
+#Same bond calculated from yield rather than from the discount curve
+yield <- 0.02
+FixedRateBond(bond,
+              coupon.rate,
+              schedule,
+              calc,
+              yield=yield)
+
+
+#same example with clean price
+price <- 103.31
+FixedRateBond(bond,
+              coupon.rate,
+              schedule,
+              calc,
+              price = price)
+
 ## bond.cpp FloatingRateBond, following test-suite/bonds.cpp
 
 bond <- list(faceAmount=100, issueDate=as.Date("2004-11-30"),
              maturityDate=as.Date("2008-11-30"), redemption=100, 
              effectiveDate=as.Date("2004-11-30"))
-dateparams <- list(settlementDays=1, calendar="us",
+dateparams <- list(settlementDays=1, calendar="UnitedStates/GovernmentBond",
                    dayCounter = 'ActualActual', period=2, 
                    businessDayConvention = 1, terminationDateConvention=1,
                    dateGeneration=0, endOfMonth=0, fixingDays = 1)
 
-gearings <- c()
-spreads <- c()
-caps <- c()
-floors <- c()
+gearings <- spreads <- caps <- floors <- vector()
 
 params <- list(tradeDate=as.Date('2002-2-15'),
                settleDate=as.Date('2002-2-19'),
